@@ -122,7 +122,7 @@ def create_home(
 
     newHome, configPath, repo = create_project(
         homedir,
-        template="home",
+        skeleton="home",
         runtime=runtime or "venv:",
         no_runtime=no_runtime,
         msg="Create the unfurl home repository",
@@ -397,7 +397,7 @@ def create_project(
     mono=False,
     existing=False,
     empty=False,
-    template=None,
+    skeleton=None,
     creating_home=False,
     **kw,
 ):
@@ -449,7 +449,7 @@ def create_project(
         repo,
         not empty and ensembleRepo,
         homePath,
-        template,
+        skeleton,
         names,
         create_context or use_context,
         mono,
@@ -1024,7 +1024,7 @@ def init_engine(projectDir, runtime):
     return "unrecognized runtime uri"
 
 
-def _run_pip_env(do_install, kw):
+def _run_pip_env(do_install, pipenv_project, kw):
     # create the virtualenv and install the dependencies specified in the Pipefiles
     sys_exit = sys.exit
     try:
@@ -1035,7 +1035,7 @@ def _run_pip_env(do_install, kw):
 
         sys.exit = noexit
 
-        do_install(**kw)
+        do_install(pipenv_project, **kw)
     finally:
         sys.exit = sys_exit
 
@@ -1082,6 +1082,7 @@ def create_venv(projectDir, pipfileLocation, unfurlLocation):
         from pipenv import environments
         from pipenv.core import do_install
         from pipenv.utils import python_version
+        from pipenv.project import Project as PipEnvProject
 
         pythonPath = os.environ["PIPENV_PYTHON"]
         assert pythonPath, pythonPath
@@ -1106,9 +1107,10 @@ def create_venv(projectDir, pipfileLocation, unfurlLocation):
                 if os.path.isfile(path):
                     shutil.copy(path, projectDir)
 
-        kw = dict(python=pythonPath)
+        kw = dict(python=pythonPath, extra_index_url=[])
+        pipenv_project = PipEnvProject()
         # need to run without args first so lock isn't overwritten
-        retcode = _run_pip_env(do_install, kw)
+        retcode = _run_pip_env(do_install, pipenv_project, kw)
         if retcode:
             return f"Pipenv (step 1) failed: {retcode}"
 
@@ -1129,7 +1131,7 @@ def create_venv(projectDir, pipfileLocation, unfurlLocation):
                 kw["packages"] = [
                     "unfurl==" + __version__()
                 ]  # use the same version as current
-        retcode = _run_pip_env(do_install, kw)
+        retcode = _run_pip_env(do_install, pipenv_project, kw)
         if retcode:
             return f"Pipenv (step 2) failed: {retcode}"
 
