@@ -262,7 +262,7 @@ def _find_template(doc, key, path, cls, fail):
             if not fail:
                 return None
             else:
-                raise UnfurlError(f"could relative path '{('.' * key.relative)}'")
+                raise UnfurlError(f"could not find relative path '{('.' * key.relative)}'")
     for index, segment in enumerate(key.pointer):
         if isinstance(template, Sequence):
             # Array indexes should be turned into integers
@@ -436,7 +436,9 @@ def expand_doc(doc, current=None, cls=dict):
             _delete_deleted_keys(expanded)
             return includes, expanded
         if len(missing) == last:  # no progress
-            raise UnfurlError(f"missing includes: {missing}")
+            raise UnfurlError(
+                "missing includes: %s" % [f"{i.key.key}:{i.value or ''}" for i in missing]
+            )
         last = len(missing)
         includes = CommentedMap()
         expanded = expand_dict(expanded, (), includes, current, cls)
