@@ -2,19 +2,19 @@
 # SPDX-License-Identifier: MIT
 import os
 import os.path
+from pathlib import Path
 import git
 from git.repo.fun import is_git_dir
 import logging
 from six.moves.urllib.parse import urlparse
-from .util import UnfurlError
-from . import getLogLevel
+from .util import UnfurlError, save_to_file, to_text
 import toscaparser.repositories
 from ruamel.yaml.comments import CommentedMap
 
 logger = logging.getLogger("unfurl")
 
 
-def normalizeGitUrl(url):
+def normalize_git_url(url):
     if url.startswith("git-local://"):  # truncate url after commit digest
         return "git-local://" + urlparse(url).netloc.partition(":")[0]
 
@@ -27,18 +27,18 @@ def normalizeGitUrl(url):
     return url
 
 
-def isURLorGitPath(url):
+def is_url_or_git_path(url):
     if "://" in url and not url.startswith("file:"):
         return True
     if "@" in url:
         return True
     candidate, sep, frag = url.partition("#")
-    if candidate.rstrip("/").endswith(".git"):
+    if frag or candidate.rstrip("/").endswith(".git"):
         return True
     return False
 
 
-def splitGitUrl(url):
+def split_git_url(url):
     """
     Returns (repoURL, filePath, revision)
     RepoURL will be an empty string if it isn't a path to a git repo
@@ -60,14 +60,14 @@ def splitGitUrl(url):
 
 class _ProgressPrinter(git.RemoteProgress):
     def update(self, op_code, cur_count, max_count=None, message=""):
-        # print update to stdout but only if logging is INFO or more verbose
-        if message and getLogLevel() <= logging.INFO:
-            print("fetching from %s, received: %s " % (self.gitUrl, message))
+        # we use print instead of logging because we don't want to clutter logs with this message
+        if message and logger.getEffectiveLevel() <= logging.INFO:
+            print(f"fetching from {self.gitUrl}, received: {message} ")
 
 
-class Repo(object):
+class Repo:
     @staticmethod
-    def findContainingRepo(rootDir, gitDir=".git"):
+    def find_containing_repo(rootDir, gitDir=".git"):
         """
         Walk parents looking for a git repository.
         """
@@ -79,44 +79,44 @@ class Repo(object):
         return None
 
     @staticmethod
-    def findGitWorkingDirs(rootDir, gitDir=".git"):
-        workingDirs = {}
+    def find_git_working_dirs(rootDir, gitDir=".git"):
+        working_dirs = {}
         for root, dirs, files in os.walk(rootDir):
-            if Repo.updateGitWorkingDirs(workingDirs, root, dirs, gitDir):
+            if Repo.update_git_working_dirs(working_dirs, root, dirs, gitDir):
                 del dirs[:]  # don't visit sub directories
-        return workingDirs
+        return working_dirs
 
     @staticmethod
-    def updateGitWorkingDirs(workingDirs, root, dirs, gitDir=".git"):
+    def update_git_working_dirs(working_dirs, root, dirs, gitDir=".git"):
         if gitDir in dirs and is_git_dir(os.path.join(root, gitDir)):
             assert os.path.isdir(root), root
             repo = GitRepo(git.Repo(root))
             key = os.path.abspath(root)
-            workingDirs[key] = repo.asRepoView()
+            working_dirs[key] = repo.as_repo_view()
             return key
         return None
 
     @staticmethod
-    def ignoreDir(dir):
-        parent = Repo.findContainingRepo(os.path.dirname(dir))
+    def ignore_dir(dir):
+        parent = Repo.find_containing_repo(os.path.dirname(dir))
         if parent:
-            path = parent.findRepoPath(dir)
+            path = parent.find_repo_path(dir)
             if path:  # can be None if dir is already ignored
-                parent.addToLocalGitIgnore(path)
+                parent.add_to_local_git_ignore("/" + path)
                 return path
         return None
 
-    def findRepoPath(self, path):
-        localPath = self.findPath(path)[0]
-        if localPath is not None and not self.isPathExcluded(localPath):
+    def find_repo_path(self, path):
+        localPath = self.find_path(path)[0]
+        if localPath is not None and not self.is_path_excluded(localPath):
             return localPath
         return None
 
-    def isPathExcluded(self, localPath):
+    def is_path_excluded(self, localPath):
         return False
 
-    def findPath(self, path, importLoader=None):
-        base = self.workingDir
+    def find_path(self, path, importLoader=None):
+        base = self.working_dir
         if not base:  # XXX support bare repos
             return None, None, None
         repoRoot = os.path.abspath(base)
@@ -128,18 +128,18 @@ class Repo(object):
             # else:
             if True:
                 revision = self.revision
-            bare = not self.workingDir or revision != self.revision
+            bare = not self.working_dir or revision != self.revision
             return abspath[len(repoRoot) + 1 :], revision, bare
         return None, None, None
 
-    def asRepoView(self, name=""):
+    def as_repo_view(self, name=""):
         return RepoView(dict(name=name, url=self.url), self)
 
-    def isLocalOnly(self):
+    def is_local_only(self):
         return self.url.startswith("git-local://") or os.path.isabs(self.url)
 
     @staticmethod
-    def getPathForGitRepo(gitUrl):
+    def get_path_for_git_repo(gitUrl):
         parts = urlparse(gitUrl)
         if parts.scheme == "git-local":
             # e.g. extract spec from git-local://0cfeee6571c4276ce1a63dc37aa8cbf8b8085d60:spec
@@ -154,13 +154,12 @@ class Repo(object):
         return name
 
     @classmethod
-    def createWorkingDir(cls, gitUrl, localRepoPath, revision=None):
+    def create_working_dir(cls, gitUrl, localRepoPath, revision=None):
         localRepoPath = localRepoPath or "."
         if os.path.exists(localRepoPath):
             if not os.path.isdir(localRepoPath) or os.listdir(localRepoPath):
                 raise UnfurlError(
-                    "couldn't create directory, it already exists and isn't empty: %s"
-                    % localRepoPath
+                    f"couldn't create directory, it already exists and isn't empty: {localRepoPath}"
                 )
         logger.info("Fetching %s %s to %s", gitUrl, revision or "", localRepoPath)
         progress = _ProgressPrinter()
@@ -172,14 +171,45 @@ class Repo(object):
             repo = git.Repo.clone_from(gitUrl, localRepoPath, progress, **kwargs)
         except git.exc.GitCommandError as err:
             raise UnfurlError(
-                'couldn\'t create working directory, clone failed: "%s"\nTry re-running that command to diagnose the problem.'
-                % err._cmdline
+                f'couldn\'t create working directory, clone failed: "{err._cmdline}"\nTry re-running that command to diagnose the problem.'
             )
-        Repo.ignoreDir(localRepoPath)
+        Repo.ignore_dir(localRepoPath)
         return GitRepo(repo)
 
 
-class RepoView(object):
+def commit_secrets(working_dir, yaml):
+    vault = yaml and getattr(yaml.representer, "vault", None)
+    if not vault:
+        return []
+    saved = []
+    for filepath, dotsecrets in find_dirty_secrets(working_dir):
+        with open(filepath, "r") as vf:
+            vaultContents = vf.read()
+        encoding = None if vaultContents.startswith("$ANSIBLE_VAULT;") else "vault"
+        secretpath = dotsecrets / filepath.name
+        logger.verbose("encrypting file to %s with %s", secretpath, vault.secrets[0][0])
+        save_to_file(str(secretpath), vaultContents, yaml, encoding)
+        saved.append(secretpath)
+    return saved
+
+
+def find_dirty_secrets(working_dir):
+    # compare .secrets with secrets
+    for root, dirs, files in os.walk(working_dir):
+        if "secrets" not in Path(root).parts:
+            continue
+        for filename in files:
+            dotsecrets = Path(root.replace("secrets", ".secrets"))
+            filepath = Path(root) / filename
+            if (
+                not dotsecrets.is_dir()
+                or filename not in list([p.name for p in dotsecrets.iterdir()])
+                or filepath.stat().st_mtime > (dotsecrets / filename).stat().st_mtime
+            ):
+                yield filepath, dotsecrets
+
+
+class RepoView:
     # view of Repo optionally filtered by path
     # XXX and revision too
     def __init__(self, repository, repo, path=""):
@@ -187,17 +217,21 @@ class RepoView(object):
             # required keys: name, url
             tpl = repository.copy()
             name = tpl.pop("name")
-            tpl["url"] = normalizeGitUrl(tpl["url"])
+            tpl["url"] = normalize_git_url(tpl["url"])
             repository = toscaparser.repositories.Repository(name, tpl)
+        assert repository or repo
         self.repository = repository
         self.repo = repo
         self.path = path
+        if repo and path and repository:
+            self.repository.url = repo.get_url_with_path(path)
         self.readOnly = not repo
+        self.yaml = None
 
     @property
-    def workingDir(self):
+    def working_dir(self):
         if self.repo:
-            return os.path.join(self.repo.workingDir, self.path)
+            return os.path.join(self.repo.working_dir, self.path)
         else:
             return os.path.join(self.repository.url, self.path)
 
@@ -209,41 +243,106 @@ class RepoView(object):
     def url(self):
         return self.repository.url if self.repository else self.repo.url
 
+    def is_local_only(self):
+        # if it doesn't have a repo then it most be local
+        return not self.repo or self.repo.is_local_only()
+
     @property
     def origin(self):
         if (
             self.repo
-            and normalizeGitUrl(self.repo.url) != self.url
-            and self.repo.url != self.repo.workingDir
+            and normalize_git_url(self.repo.url) != split_git_url(self.url)[0]
+            and self.repo.url != self.repo.working_dir
         ):
             return self.repo.url
         return ""
 
-    def isDirty(self):
+    def is_dirty(self):
         if self.readOnly:
             return False
-        return self.repo.isDirty(untracked_files=True, path=self.path)
+        for filepath, dotsecrets in find_dirty_secrets(self.working_dir):
+            return True
+        return self.repo.is_dirty(untracked_files=True, path=self.path)
 
-    def addAll(self):
+    def add_all(self):
+        assert not self.readOnly
         self.repo.repo.git.add("--all", self.path or ".")
 
+    def load_secrets(self, _loader):
+        logger.trace("looking for secrets %s", self.working_dir)
+        for root, dirs, files in os.walk(self.working_dir):
+            if ".secrets" not in Path(root).parts:
+                continue
+            logger.trace("checking if secret files where changed or added %s", files)
+            for filename in files:
+                secretsdir = Path(root.replace(".secrets", "secrets"))
+                filepath = Path(root) / filename
+                stinfo = filepath.stat()
+                target = secretsdir / filename
+                if not target.is_file() or stinfo.st_mtime > target.stat().st_mtime:
+                    target = secretsdir / filename
+                    try:
+                        contents = _loader.load_from_file(str(filepath))
+                    except Exception as err:
+                        logger.warning("could not decrypt %s: %s", filepath, err)
+                        continue
+                    with open(str(target), "w") as f:
+                        f.write(contents)
+                    os.utime(target, (stinfo.st_atime, stinfo.st_mtime))
+                    logger.verbose("decrypted secret file to %s", target)
+
+    def save_secrets(self):
+        return commit_secrets(self.working_dir, self.yaml)
+
     def commit(self, message, addAll=False):
+        assert not self.readOnly
+        if self.yaml:
+            for saved in self.save_secrets():
+                self.repo.repo.git.add(str(saved.relative_to(self.repo.working_dir)))
         if addAll:
-            self.addAll()
-        return self.repo.repo.index.commit(message)
+            self.add_all()
+        self.repo.repo.index.commit(message)
+        return 1
 
-    def status(self):
-        return self.repo.runCmd(["status", self.path or "."])[1]
+    def get_default_commit_message(self):
+        return "Commit by Unfurl"
 
-    def getInitialRevision(self):
+    def git_status(self):
+        assert not self.readOnly
+        return self.repo.run_cmd(["status", self.path or "."])[1]
+
+    def _secrets_status(self):
+        modified = "\n   ".join(
+            [
+                str(filepath.relative_to(self.repo.working_dir))
+                for filepath, dotsecrets in find_dirty_secrets(self.working_dir)
+            ]
+        )
+        if modified:
+            return f"\n\nSecrets to be committed:\n   {modified}"
+        return ""
+
+    def get_repo_status(self, dirty=False):
+        if self.repo and (not dirty or self.is_dirty()):
+            git_status = self.git_status()
+            if self.name:
+                header = f"for {self.name} at {self.working_dir}"
+            else:
+                header = f"for {self.working_dir}"
+            secrets_status = self._secrets_status()
+            return f"Status {header}:\n{git_status}{secrets_status}\n\n"
+        else:
+            return ""
+
+    def get_initial_revision(self):
         if not self.repo:
             return ""
-        return self.repo.getInitialRevision()
+        return self.repo.get_initial_revision()
 
-    def getCurrentRevision(self):
+    def get_current_revision(self):
         if not self.repo:
             return ""
-        if self.isDirty():
+        if self.is_dirty():
             return self.repo.revision + "-dirty"
         else:
             return self.repo.revision
@@ -251,12 +350,13 @@ class RepoView(object):
     def lock(self):
         record = CommentedMap(
             [
-                ("name", self.name),
                 ("url", self.url),
-                ("revision", self.getCurrentRevision()),
-                ("initial", self.getInitialRevision()),
+                ("revision", self.get_current_revision()),
+                ("initial", self.get_initial_revision()),
             ]
         )
+        if self.name:
+            record["name"] = self.name
         if self.origin:
             record["origin"] = self.origin
         return record
@@ -265,7 +365,7 @@ class RepoView(object):
 class GitRepo(Repo):
     def __init__(self, gitrepo):
         self.repo = gitrepo
-        self.url = self.workingDir or gitrepo.git_dir
+        self.url = self.working_dir or gitrepo.git_dir
         if gitrepo.remotes:
             # note: these might not look like absolute urls, e.g. git@github.com:onecommons/unfurl.git
             try:
@@ -275,7 +375,7 @@ class GitRepo(Repo):
             self.url = remote.url
 
     @property
-    def workingDir(self):
+    def working_dir(self):
         dir = self.repo.working_tree_dir
         if not dir or dir[-1] == "/":
             return dir
@@ -288,24 +388,24 @@ class GitRepo(Repo):
             return ""
         return self.repo.head.commit.hexsha
 
-    def resolveRevSpec(self, revision):
+    def resolve_rev_spec(self, revision):
         try:
             return self.repo.commit(revision).hexsha
         except:
             return None
 
-    def getUrlWithPath(self, path):
-        if isURLorGitPath(self.url):
+    def get_url_with_path(self, path):
+        if is_url_or_git_path(self.url):
             if os.path.isabs(path):
                 # get path relative to repository's root
-                path = os.path.relpath(path, self.workingDir)
-            return normalizeGitUrl(self.url) + "#:" + path
+                path = os.path.relpath(path, self.working_dir)
+            return normalize_git_url(self.url) + "#:" + path
         else:
-            return self.getGitLocalUrl(path)
+            return self.get_git_local_url(path)
 
-    def findExcludedDirs(self, root):
-        root = os.path.relpath(root, self.workingDir)
-        status, stdout, stderr = self.runCmd(
+    def find_excluded_dirs(self, root):
+        root = os.path.relpath(root, self.working_dir)
+        status, stdout, stderr = self.run_cmd(
             [
                 "ls-files",
                 "--exclude-standard",
@@ -317,16 +417,16 @@ class GitRepo(Repo):
             ]
         )
         for file in stdout.splitlines():
-            path = os.path.join(self.workingDir, file)
+            path = os.path.join(self.working_dir, file)
             yield path
 
-    def isPathExcluded(self, localPath):
+    def is_path_excluded(self, localPath):
         # XXX cache and test
-        # excluded = list(self.findExcludedDirs(self.workingDir))
+        # excluded = list(self.findExcludedDirs(self.working_dir))
         # success error code means it's ignored
-        return not self.runCmd(["check-ignore", "-q", localPath])[0]
+        return not self.run_cmd(["check-ignore", "-q", localPath])[0]
 
-    def runCmd(self, args, **kw):
+    def run_cmd(self, args, **kw):
         """
         :return:
           tuple(int(status), str(stdout), str(stderr))
@@ -342,13 +442,13 @@ class GitRepo(Repo):
             call, with_exceptions=False, with_extended_output=True, **kw
         )
 
-    def addToLocalGitIgnore(self, rule):
+    def add_to_local_git_ignore(self, rule):
         with open(os.path.join(self.repo.git_dir, "info", "exclude"), "a") as f:
             f.write("\n" + rule + "\n")
 
     def show(self, path, commitId):
-        if self.workingDir and os.path.isabs(path):
-            path = os.path.abspath(path)[len(self.workingDir) :]
+        if self.working_dir and os.path.isabs(path):
+            path = os.path.abspath(path)[len(self.working_dir) :]
         # XXX this won't work if path is in a submodule
         # if in path startswith a submodule: git log -1 -p [commitid] --  [submodule]
         # submoduleCommit = re."\+Subproject commit (.+)".group(1)
@@ -363,13 +463,13 @@ class GitRepo(Repo):
             "checking out '%s' at %s to %s",
             self.url,
             revision or "HEAD",
-            self.workingDir,
+            self.working_dir,
         )
-        return self.workingDir
+        return self.working_dir
 
-    def addSubModule(self, gitDir):
+    def add_sub_module(self, gitDir):
         gitDir = os.path.abspath(gitDir)
-        status, stdout, stderr = self.runCmd(["submodule", "add", gitDir])
+        status, stdout, stderr = self.run_cmd(["submodule", "add", gitDir])
         success = not status
         if success:
             logging.debug("added submodule %s: %s %s", gitDir, stdout, stderr)
@@ -377,23 +477,23 @@ class GitRepo(Repo):
             logging.error("failed to add submodule %s: %s %s", gitDir, stdout, stderr)
         return success
 
-    def getInitialRevision(self):
+    def get_initial_revision(self):
         if not self.repo.head.is_valid():
             return ""  # an uninitialized repo
         firstCommit = next(self.repo.iter_commits("HEAD", max_parents=0))
         return firstCommit.hexsha
 
-    def addAll(self, path="."):
-        path = os.path.relpath(path, self.workingDir)
+    def add_all(self, path="."):
+        path = os.path.relpath(path, self.working_dir)
         self.repo.git.add("--all", path)
 
-    def commitFiles(self, files, msg):
+    def commit_files(self, files, msg):
         # note: this will also commit existing changes in the index
         index = self.repo.index
         index.add([os.path.abspath(f) for f in files])
         return index.commit(msg)
 
-    def isDirty(self, untracked_files=False, path=None):
+    def is_dirty(self, untracked_files=False, path=None):
         # diff = self.repo.git.diff()  # "--abbrev=40", "--full-index", "--raw")
         # https://gitpython.readthedocs.io/en/stable/reference.html?highlight=is_dirty#git.repo.base.Repo.is_dirty
         return self.repo.is_dirty(untracked_files=untracked_files, path=path or None)
@@ -401,16 +501,16 @@ class GitRepo(Repo):
     def clone(self, newPath):
         # note: repo.clone uses bare path, which breaks submodule path resolution
         cloned = git.Repo.clone_from(
-            self.workingDir, os.path.abspath(newPath), recurse_submodules=True
+            self.working_dir, os.path.abspath(newPath), recurse_submodules=True
         )
-        Repo.ignoreDir(newPath)
+        Repo.ignore_dir(newPath)
         return GitRepo(cloned)
 
-    def getGitLocalUrl(self, path, name=""):
+    def get_git_local_url(self, path, name=""):
         if os.path.isabs(path):
             # get path relative to repository's root
-            path = os.path.relpath(path, self.workingDir)
-        return "git-local://%s:%s/%s" % (self.getInitialRevision(), name, path)
+            path = os.path.relpath(path, self.working_dir)
+        return f"git-local://{self.get_initial_revision()}:{name}/{path}"
 
     # XXX: def getDependentRepos()
     # XXX: def canManage()
@@ -453,13 +553,13 @@ class GitRepo(Repo):
     #     repo.git.commit("")
 
 
-class RevisionManager(object):
+class RevisionManager:
     def __init__(self, manifest, localEnv=None):
         self.manifest = manifest
         self.revisions = None
         self.localEnv = localEnv
 
-    def getRevision(self, change):
+    def get_revision(self, change):
         if self.revisions is None:
             self.revisions = {self.manifest.specDigest: self.manifest}
         digest = change["specDigest"]
