@@ -6,7 +6,7 @@ TOSCA implementation
 import functools
 import copy
 from .tosca_plugins import TOSCA_VERSION
-from .util import UnfurlError, UnfurlValidationError, get_base_dir
+from .util import UnfurlError, UnfurlValidationError, get_base_dir, check_class_registry
 from .eval import Ref, RefContext, map_value
 from .result import ResourceRef, ResultsList
 from .merge import patch_dict, merge_dicts
@@ -174,7 +174,6 @@ class ToscaSpec:
             import_resolver=resolver,
             verify=False,  # we display the error messages ourselves so we don't need to verify here
         )
-
         self.nodeTemplates = {}
         self.installers = {}
         self.relationshipTemplates = {}
@@ -200,11 +199,7 @@ class ToscaSpec:
         }
 
     def __init__(
-        self,
-        toscaDef,
-        spec=None,
-        path=None,
-        resolver=None,
+        self, toscaDef, spec=None, path=None, resolver=None, skip_validation=False
     ):
         self.discovered = None
         if spec:
@@ -238,7 +233,7 @@ class ToscaSpec:
                 # copy errors before we clear them in _overlay
                 errorsSoFar = ExceptionCollector.exceptions[:]
                 self._overlay(decorators)
-                if ExceptionCollector.exceptionsCaught():
+                if not skip_validation and ExceptionCollector.exceptionsCaught():
                     # abort if overlay caused errors
                     # report previously collected errors too
                     ExceptionCollector.exceptions[:0] = errorsSoFar
@@ -256,7 +251,7 @@ class ToscaSpec:
                 # overlay and evaluate_imports modifies tosaDef in-place, try reparsing it
                 self._parse_template(path, inputs, toscaDef, resolver)
 
-            if ExceptionCollector.exceptionsCaught():
+            if not skip_validation and ExceptionCollector.exceptionsCaught():
                 message = "\n".join(
                     ExceptionCollector.getExceptionsReport(
                         full=(get_console_log_level() < logging.INFO)
@@ -585,7 +580,7 @@ class EntitySpec(ResourceRef):
             # i.e. missing properties without a default and not required
             props_def = toscaNodeTemplate.type_definition.get_properties_def()
             for pDef in props_def.values():
-                if pDef.name not in self.propertyDefs:
+                if pDef.schema and pDef.name not in self.propertyDefs:
                     self.propertyDefs[pDef.name] = Property(
                         pDef.name,
                         pDef.default,
@@ -653,7 +648,7 @@ class EntitySpec(ResourceRef):
         else:
             return name
 
-    def find_or_create_artifact(self, nameOrTpl, path=None):
+    def find_or_create_artifact(self, nameOrTpl, path=None, predefined=False):
         if not nameOrTpl:
             return None
         if isinstance(nameOrTpl, six.string_types):
@@ -684,7 +679,11 @@ class EntitySpec(ResourceRef):
                     tpl["repository"] = localStore.name
                     break
             else:
-                # name not found, assume it's a file path or URL
+                if predefined and not check_class_registry(name):
+                    logger.warning(f"no artifact named {name} found")
+                    return None
+
+                # otherwise name not found, assume it's a file path or URL
                 tpl = dict(file=name)
         else:
             # see if this is declared in a repository node template with the same name
