@@ -294,7 +294,13 @@ class YamlManifest(ReadOnlyManifest):
     lockfile = None
 
     def __init__(
-        self, manifest=None, path=None, validate=True, localEnv=None, vault=None
+        self,
+        manifest=None,
+        path=None,
+        validate=True,
+        localEnv=None,
+        vault=None,
+        skip_validation=False,
     ):
         super().__init__(manifest, path, validate, localEnv, vault)
         # instantiate the tosca template
@@ -303,7 +309,7 @@ class YamlManifest(ReadOnlyManifest):
             self.lockfilepath = self.manifest.path + ".lock"
 
         more_spec = self._load_context(self.context, localEnv)
-        self._set_spec(manifest, more_spec)
+        self._set_spec(manifest, more_spec, skip_validation)
         assert self.tosca
         spec = manifest.get("spec", {})
         status = manifest.get("status", {})
@@ -598,8 +604,9 @@ class YamlManifest(ReadOnlyManifest):
         return self.save_entity_instance(resource)
 
     def save_resource(self, resource, discovered):
-        # XXX checkstatus break unit tests:
-        ret = self._save_entity_if_instantiated(resource, False)
+        # XXX checkstatus break unit tests so skip mostly
+        checkstatus = resource.template.type == "unfurl.nodes.LocalRepository"
+        ret = self._save_entity_if_instantiated(resource, checkstatus)
         if not ret:
             return ret
         name, status = ret
